@@ -1,7 +1,7 @@
 import tkinter
 import sys
-from math import ceil, floor
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
@@ -10,56 +10,82 @@ from .customtkinter_color_manager import CTkColorManager
 class CTkButton(tkinter.Frame):
     """ tkinter custom button with border, rounded corners and hover effect """
 
-    def __init__(self,
+    def __init__(self, *args,
                  bg_color=None,
-                 fg_color=CTkColorManager.MAIN,
-                 hover_color=CTkColorManager.MAIN_HOVER,
+                 fg_color="CTkColorManager",
+                 hover_color="CTkColorManager",
                  border_color=None,
                  border_width=0,
                  command=None,
+                 textvariable=None,
                  width=120,
-                 height=32,
+                 height=30,
                  corner_radius=8,
                  text_font=None,
-                 text_color=CTkColorManager.TEXT,
+                 text_color="CTkColorManager",
                  text="CTkButton",
                  hover=True,
                  image=None,
                  compound=tkinter.LEFT,
                  state=tkinter.NORMAL,
-                 *args, **kwargs):
+                 **kwargs):
         super().__init__(*args, **kwargs)
 
-        AppearanceModeTracker.add(self.set_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribute>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.set_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
         self.configure_basic_grid()
 
+        # color variables
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.fg_color = self.bg_color if fg_color is None else fg_color
-        self.fg_color = self.bg_color if self.fg_color is None else self.fg_color
-        self.hover_color = self.fg_color if hover_color is None else hover_color
+        self.fg_color = CTkColorManager.MAIN if fg_color == "CTkColorManager" else fg_color
+        self.hover_color = CTkColorManager.MAIN_HOVER if hover_color == "CTkColorManager" else hover_color
         self.border_color = border_color
 
+        # shape and size
         self.width = width
         self.height = height
-
+        self.configure(width=self.width, height=self.height)
         self.corner_radius = self.calc_optimal_corner_radius(corner_radius)  # optimise for less artifacts
+        self.border_width = round(border_width)  # round border_width (inner parts not centered otherwise)
 
         if self.corner_radius * 2 > self.height:
             self.corner_radius = self.height / 2
         elif self.corner_radius * 2 > self.width:
             self.corner_radius = self.width / 2
 
-        self.border_width = round(border_width)  # round border_width (inner parts not centered otherwise)
-
         if self.corner_radius >= self.border_width:
             self.inner_corner_radius = self.corner_radius - self.border_width
         else:
             self.inner_corner_radius = 0
 
+        # text and font and image
+        self.image = image
+        self.image_label = None
         self.text = text
-        self.text_color = text_color
+        self.text_label = None
+        self.text_color = CTkColorManager.TEXT if text_color == "CTkColorManager" else text_color
         if text_font is None:
             if sys.platform == "darwin":  # macOS
                 self.text_font = ("Avenir", 13)
@@ -70,13 +96,13 @@ class CTkButton(tkinter.Frame):
         else:
             self.text_font = text_font
 
+        # callback and hover functionality
         self.function = command
+        self.textvariable = textvariable
         self.state = state
         self.hover = hover
-        self.image = image
         self.compound = compound
-
-        self.configure(width=self.width, height=self.height)
+        self.click_animation_running = False
 
         if sys.platform == "darwin" and self.function is not None:
             self.configure(cursor="pointinghand")  # other cursor when hovering over button with command
@@ -87,6 +113,7 @@ class CTkButton(tkinter.Frame):
                                      height=self.height)
         self.canvas.grid(row=0, column=0, rowspan=2, columnspan=2)
 
+        # event bindings
         if self.hover is True:
             self.canvas.bind("<Enter>", self.on_enter)
             self.canvas.bind("<Leave>", self.on_leave)
@@ -94,12 +121,10 @@ class CTkButton(tkinter.Frame):
         self.canvas.bind("<Button-1>", self.clicked)
         self.canvas.bind("<Button-1>", self.clicked)
 
-        self.text_label = None
-        self.image_label = None
-
         # Each time an item is resized due to pack position mode, the binding Configure is called on the widget
         self.bind('<Configure>', self.update_dimensions)
-        self.draw()
+
+        self.draw()  # initial draw
 
     def destroy(self):
         AppearanceModeTracker.remove(self.set_appearance_mode)
@@ -113,13 +138,12 @@ class CTkButton(tkinter.Frame):
         self.grid_columnconfigure(1, weight=1)
 
     def update_dimensions(self, event):
-        # We update the dimensions of the internal elements of CTkButton Widget
-        self.canvas.config(width=event.width, height=event.height)
-
         # only redraw if dimensions changed (for performance)
         if self.width != event.width or self.height != event.height:
             self.width = event.width
             self.height = event.height
+
+            self.canvas.config(width=self.width, height=self.height)
             self.draw(no_color_updates=True)  # fast drawing without color changes
 
     def detect_color_of_master(self):
@@ -163,20 +187,32 @@ class CTkButton(tkinter.Frame):
 
             # set color for inner button parts (depends on button state)
             if self.state == tkinter.DISABLED:
-                self.canvas.itemconfig("inner_parts",
-                                       outline=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.fg_color, self.appearance_mode)),
-                                       fill=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.fg_color, self.appearance_mode)))
+                if self.fg_color is None:
+                    self.canvas.itemconfig("inner_parts",
+                                           outline=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.bg_color, self.appearance_mode)),
+                                           fill=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.bg_color, self.appearance_mode)))
+                else:
+                    self.canvas.itemconfig("inner_parts",
+                                           outline=CTkColorManager.darken_hex_color(
+                                                   CTkColorManager.single_color(self.fg_color, self.appearance_mode)),
+                                           fill=CTkColorManager.darken_hex_color(
+                                                   CTkColorManager.single_color(self.fg_color, self.appearance_mode)))
 
             else:
-                self.canvas.itemconfig("inner_parts",
-                                       outline=CTkColorManager.single_color(self.fg_color, self.appearance_mode),
-                                       fill=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+                if self.fg_color is None:
+                    self.canvas.itemconfig("inner_parts",
+                                           outline=CTkColorManager.single_color(self.bg_color, self.appearance_mode),
+                                           fill=CTkColorManager.single_color(self.bg_color, self.appearance_mode))
+                else:
+                    self.canvas.itemconfig("inner_parts",
+                                           outline=CTkColorManager.single_color(self.fg_color, self.appearance_mode),
+                                           fill=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
 
         # create text label if text given
         if self.text is not None and self.text != "":
 
             if self.text_label is None:
-                self.text_label = tkinter.Label(master=self, font=self.text_font)
+                self.text_label = tkinter.Label(master=self, font=self.text_font, textvariable=self.textvariable)
 
                 self.text_label.bind("<Enter>", self.on_enter)
                 self.text_label.bind("<Leave>", self.on_leave)
@@ -189,9 +225,17 @@ class CTkButton(tkinter.Frame):
 
                 # set text_label bg color (label color)
                 if self.state == tkinter.DISABLED:
-                    self.text_label.configure(bg=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.fg_color, self.appearance_mode)))
+                    if self.fg_color is None:
+                        self.text_label.configure(
+                                bg=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.bg_color, self.appearance_mode)))
+                    else:
+                        self.text_label.configure(
+                                bg=CTkColorManager.darken_hex_color(CTkColorManager.single_color(self.fg_color, self.appearance_mode)))
                 else:
-                    self.text_label.configure(bg=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+                    if self.fg_color is None:
+                        self.text_label.configure(bg=CTkColorManager.single_color(self.bg_color, self.appearance_mode))
+                    else:
+                        self.text_label.configure(bg=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
 
             self.text_label.configure(text=self.text)  # set text
 
@@ -422,6 +466,11 @@ class CTkButton(tkinter.Frame):
             require_redraw = True
             del kwargs["fg_color"]
 
+        if "border_color" in kwargs:
+            self.border_color = kwargs["border_color"]
+            require_redraw = True
+            del kwargs["border_color"]
+
         if "bg_color" in kwargs:
             if kwargs["bg_color"] is None:
                 self.bg_color = self.detect_color_of_master()
@@ -440,6 +489,16 @@ class CTkButton(tkinter.Frame):
             require_redraw = True
             del kwargs["text_color"]
 
+        if "command" in kwargs:
+            self.function = kwargs["command"]
+            del kwargs["command"]
+
+        if "textvariable" in kwargs:
+            self.textvariable = kwargs["textvariable"]
+            if self.text_label is not None:
+                self.text_label.configure(textvariable=self.textvariable)
+            del kwargs["textvariable"]
+
         super().configure(*args, **kwargs)
 
         if require_redraw:
@@ -470,39 +529,60 @@ class CTkButton(tkinter.Frame):
 
     def on_enter(self, event=0):
         if self.hover is True:
+            if self.hover_color is None:
+                inner_parts_color = self.fg_color
+            else:
+                inner_parts_color = self.hover_color
+
             # set color of inner button parts to hover color
             self.canvas.itemconfig("inner_parts",
-                                   outline=CTkColorManager.single_color(self.hover_color, self.appearance_mode),
-                                   fill=CTkColorManager.single_color(self.hover_color, self.appearance_mode))
+                                   outline=CTkColorManager.single_color(inner_parts_color, self.appearance_mode),
+                                   fill=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
 
             # set text_label bg color to button hover color
             if self.text_label is not None:
-                self.text_label.configure(bg=CTkColorManager.single_color(self.hover_color, self.appearance_mode))
+                self.text_label.configure(bg=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
 
             # set image_label bg color to button hover color
             if self.image_label is not None:
-                self.image_label.configure(bg=CTkColorManager.single_color(self.hover_color, self.appearance_mode))
+                self.image_label.configure(bg=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
 
     def on_leave(self, event=0):
+        self.click_animation_running = False
+
         if self.hover is True:
+            if self.fg_color is None:
+                inner_parts_color = self.bg_color
+            else:
+                inner_parts_color = self.fg_color
+
             # set color of inner button parts
             self.canvas.itemconfig("inner_parts",
-                                   outline=CTkColorManager.single_color(self.fg_color, self.appearance_mode),
-                                   fill=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+                                   outline=CTkColorManager.single_color(inner_parts_color, self.appearance_mode),
+                                   fill=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
 
             # set text_label bg color (label color)
             if self.text_label is not None:
-                self.text_label.configure(bg=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+                self.text_label.configure(bg=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
 
             # set image_label bg color (image bg color)
             if self.image_label is not None:
-                self.image_label.configure(bg=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+                self.image_label.configure(bg=CTkColorManager.single_color(inner_parts_color, self.appearance_mode))
+
+    def click_animation(self):
+        if self.click_animation_running:
+            self.on_enter()
 
     def clicked(self, event=0):
         if self.function is not None:
             if self.state is not tkinter.DISABLED:
-                self.function()
+
+                # click animation: change color with .on_leave() and back to normal after 100ms with click_animation()
                 self.on_leave()
+                self.click_animation_running = True
+                self.after(100, self.click_animation)
+
+                self.function()
 
     def set_appearance_mode(self, mode_string):
         if mode_string.lower() == "dark":
@@ -510,4 +590,9 @@ class CTkButton(tkinter.Frame):
         elif mode_string.lower() == "light":
             self.appearance_mode = 0
 
+        if isinstance(self.master, (CTkFrame, CTk)):
+            self.bg_color = self.master.fg_color
+        else:
+            self.bg_color = self.master.cget("bg")
+
         self.draw()
