@@ -1,6 +1,7 @@
 import tkinter
 import sys
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
@@ -9,31 +10,56 @@ from .customtkinter_color_manager import CTkColorManager
 class CTkCheckBox(tkinter.Frame):
     """ tkinter custom checkbox with border, rounded corners and hover effect """
 
-    def __init__(self,
+    def __init__(self, *args,
                  bg_color=None,
-                 fg_color=CTkColorManager.MAIN,
-                 hover_color=CTkColorManager.MAIN_HOVER,
-                 border_color=CTkColorManager.CHECKBOX_LINES,
+                 fg_color="CTkColorManager",
+                 hover_color="CTkColorManager",
+                 border_color="CTkColorManager",
                  border_width=3,
                  width=25,
                  height=25,
                  corner_radius=4,
                  text_font=None,
-                 text_color=CTkColorManager.TEXT,
+                 text_color="CTkColorManager",
                  text="CTkCheckBox",
                  hover=True,
                  command=None,
                  state=tkinter.NORMAL,
-                 *args, **kwargs):
+                 onvalue=1,
+                 offvalue=0,
+                 variable=None,
+                 textvariable=None,
+                 **kwargs):
         super().__init__(*args, **kwargs)
 
-        AppearanceModeTracker.add(self.set_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribut>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.set_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.fg_color = CTkColorManager.MAIN if fg_color is None else fg_color
-        self.hover_color = CTkColorManager.MAIN_HOVER if hover_color is None else hover_color
-        self.border_color = border_color
+        self.fg_color = CTkColorManager.MAIN if fg_color == "CTkColorManager" else fg_color
+        self.hover_color = CTkColorManager.MAIN_HOVER if hover_color == "CTkColorManager" else hover_color
+        self.border_color = CTkColorManager.CHECKBOX_LINES if border_color == "CTkColorManager" else border_color
 
         self.width = width
         self.height = height
@@ -52,7 +78,7 @@ class CTkCheckBox(tkinter.Frame):
             self.inner_corner_radius = 0
 
         self.text = text
-        self.text_color = text_color
+        self.text_color = CTkColorManager.TEXT if text_color == "CTkColorManager" else text_color
         if text_font is None:
             if sys.platform == "darwin":  # macOS
                 self.text_font = ("Avenir", 13)
@@ -67,6 +93,12 @@ class CTkCheckBox(tkinter.Frame):
         self.state = state
         self.hover = hover
         self.check_state = False
+        self.onvalue = onvalue
+        self.offvalue = offvalue
+        self.variable: tkinter.Variable = variable
+        self.variable_callback_blocked = False
+        self.textvariable = textvariable
+        self.variabel_callback_name = None
 
         self.canvas = tkinter.Canvas(master=self,
                                      highlightthicknes=0,
@@ -89,10 +121,21 @@ class CTkCheckBox(tkinter.Frame):
         self.canvas_check_parts = []
         self.text_label = None
 
-        self.draw()
+        self.draw()  # initial draw
+
+        if self.variable is not None:
+            self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+            if self.variable.get() == self.onvalue:
+                self.select(from_variable_callback=True)
+            elif self.variable.get() == self.offvalue:
+                self.deselect(from_variable_callback=True)
 
     def destroy(self):
         AppearanceModeTracker.remove(self.set_appearance_mode)
+
+        if self.variable is not None:
+            self.variable.trace_remove("write", self.variabel_callback_name)
+
         super().destroy()
 
     def detect_color_of_master(self):
@@ -191,11 +234,18 @@ class CTkCheckBox(tkinter.Frame):
                                                                  self.width - self.border_width,
                                                                  self.height - self.inner_corner_radius - self.border_width))
 
-        for part in self.canvas_fg_parts:
-            if type(self.bg_color) == tuple and len(self.bg_color) == 2:
-                self.canvas.itemconfig(part, fill=self.bg_color[self.appearance_mode], width=0)
-            else:
-                self.canvas.itemconfig(part, fill=self.bg_color, outline=self.bg_color, width=0)
+        if self.check_state is False:
+            for part in self.canvas_fg_parts:
+                if type(self.bg_color) == tuple and len(self.bg_color) == 2:
+                        self.canvas.itemconfig(part, fill=self.bg_color[self.appearance_mode], width=0)
+                else:
+                        self.canvas.itemconfig(part, fill=self.bg_color, outline=self.bg_color, width=0)
+        else:
+            for part in self.canvas_fg_parts:
+                if type(self.fg_color) == tuple and len(self.fg_color) == 2:
+                        self.canvas.itemconfig(part, fill=self.fg_color[self.appearance_mode], width=0)
+                else:
+                        self.canvas.itemconfig(part, fill=self.fg_color, outline=self.bg_color, width=0)
 
         for part in self.canvas_border_parts:
             if type(self.border_color) == tuple and len(self.border_color) == 2:
@@ -267,6 +317,27 @@ class CTkCheckBox(tkinter.Frame):
             require_redraw = True
             del kwargs["border_color"]
 
+        if "command" in kwargs:
+            self.function = kwargs["command"]
+            del kwargs["command"]
+
+        if "variable" in kwargs:
+            if self.variable is not None:
+                self.variable.trace_remove("write", self.variabel_callback_name)
+
+            self.variable = kwargs["variable"]
+
+            if self.variable is not None and self.variable != "":
+                self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+                if self.variable.get() == self.onvalue:
+                    self.select(from_variable_callback=True)
+                elif self.variable.get() == self.offvalue:
+                    self.deselect(from_variable_callback=True)
+            else:
+                self.variable = None
+
+            del kwargs["variable"]
+
         super().configure(*args, **kwargs)
 
         if require_redraw:
@@ -317,6 +388,13 @@ class CTkCheckBox(tkinter.Frame):
                     else:
                         self.canvas.itemconfig(part, fill=self.bg_color, width=0)
 
+    def variable_callback(self, var_name, index, mode):
+        if not self.variable_callback_blocked:
+            if self.variable.get() == self.onvalue:
+                self.select(from_variable_callback=True)
+            elif self.variable.get() == self.offvalue:
+                self.deselect(from_variable_callback=True)
+
     def toggle(self, event=0):
         if self.state == tkinter.NORMAL:
             if self.check_state is True:
@@ -337,7 +415,12 @@ class CTkCheckBox(tkinter.Frame):
             if self.function is not None:
                 self.function()
 
-    def select(self):
+            if self.variable is not None:
+                self.variable_callback_blocked = True
+                self.variable.set(self.onvalue if self.check_state is True else self.offvalue)
+                self.variable_callback_blocked = False
+
+    def select(self, from_variable_callback=False):
         self.check_state = True
         for part in self.canvas_fg_parts:
             if type(self.fg_color) == tuple and len(self.fg_color) == 2:
@@ -348,7 +431,12 @@ class CTkCheckBox(tkinter.Frame):
         if self.function is not None:
             self.function()
 
-    def deselect(self):
+        if self.variable is not None and not from_variable_callback:
+            self.variable_callback_blocked = True
+            self.variable.set(self.onvalue)
+            self.variable_callback_blocked = False
+
+    def deselect(self, from_variable_callback=False):
         self.check_state = False
         for part in self.canvas_fg_parts:
             if type(self.bg_color) == tuple and len(self.bg_color) == 2:
@@ -359,8 +447,13 @@ class CTkCheckBox(tkinter.Frame):
         if self.function is not None:
             self.function()
 
+        if self.variable is not None and not from_variable_callback:
+            self.variable_callback_blocked = True
+            self.variable.set(self.offvalue)
+            self.variable_callback_blocked = False
+
     def get(self):
-        return 1 if self.check_state is True else 0
+        return self.onvalue if self.check_state is True else self.offvalue
 
     def set_appearance_mode(self, mode_string):
         if mode_string.lower() == "dark":
@@ -368,5 +461,9 @@ class CTkCheckBox(tkinter.Frame):
         elif mode_string.lower() == "light":
             self.appearance_mode = 0
 
-        self.draw()
+        if isinstance(self.master, (CTkFrame, CTk)):
+            self.bg_color = self.master.fg_color
+        else:
+            self.bg_color = self.master.cget("bg")
 
+        self.draw()
