@@ -1,30 +1,56 @@
 import tkinter
 import sys
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
 
 
 class CTkEntry(tkinter.Frame):
-    def __init__(self,
+    def __init__(self, *args,
                  master=None,
                  bg_color=None,
-                 fg_color=CTkColorManager.ENTRY,
-                 text_color=CTkColorManager.TEXT,
-                 corner_radius=10,
+                 fg_color="CTkColorManager",
+                 text_color="CTkColorManager",
+                 corner_radius=8,
                  width=120,
-                 height=25,
-                 *args,
+                 height=30,
                  **kwargs):
-        super().__init__(master=master)
+        if master is None:
+            super().__init__(*args)
+        else:
+            super().__init__(*args, master=master)
 
-        AppearanceModeTracker.add(self.change_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribut>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.change_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
+        self.configure_basic_grid()
+
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.fg_color = self.bg_color if fg_color is None else fg_color
-        self.text_color = text_color
+        self.fg_color = CTkColorManager.ENTRY if fg_color == "CTkColorManager" else fg_color
+        self.text_color = CTkColorManager.TEXT if text_color == "CTkColorManager" else text_color
 
         self.width = width
         self.height = height
@@ -36,28 +62,34 @@ class CTkEntry(tkinter.Frame):
         elif self.corner_radius*2 > self.width:
             self.corner_radius = self.width/2
 
-        self.configure(width=self.width, height=self.height)
+        super().configure(width=self.width, height=self.height)
 
         self.canvas = tkinter.Canvas(master=self,
                                      highlightthicknes=0,
                                      width=self.width,
                                      height=self.height)
-        self.canvas.place(x=0, y=0)
+        self.canvas.grid(column=0, row=0)
 
         self.entry = tkinter.Entry(master=self,
                                    bd=0,
+                                   width=1,
                                    highlightthicknes=0,
-                                   *args, **kwargs)
-        self.entry.place(relx=0.5, rely=0.5, relwidth=0.8, anchor=tkinter.CENTER)
+                                   **kwargs)
+        self.entry.grid(column=0, row=0, sticky="we", padx=self.corner_radius if self.corner_radius >= 6 else 6)
 
         self.fg_parts = []
 
+        self.bind('<Configure>', self.update_dimensions)
         self.draw()
 
     def destroy(self):
         AppearanceModeTracker.remove(self.change_appearance_mode)
         super().destroy()
 
+    def configure_basic_grid(self):
+        self.grid_rowconfigure(0, weight=1)
+        self.grid_columnconfigure(0, weight=1)
+
     def detect_color_of_master(self):
         if isinstance(self.master, CTkFrame):
             return self.master.fg_color
@@ -79,6 +111,15 @@ class CTkEntry(tkinter.Frame):
             else:
                 return user_corner_radius
 
+    def update_dimensions(self, event):
+        # only redraw if dimensions changed (for performance)
+        if self.width != event.width or self.height != event.height:
+            self.width = event.width
+            self.height = event.height
+
+            self.canvas.config(width=self.width, height=self.height)
+            self.draw()
+
     def draw(self):
         self.canvas.delete("all")
         self.fg_parts = []
@@ -140,20 +181,39 @@ class CTkEntry(tkinter.Frame):
         self.configure(*args, **kwargs)
 
     def configure(self, *args, **kwargs):
-        super().configure(*args, **kwargs)
+        require_redraw = False  # some attribute changes require a call of self.draw() at the end
 
-    def change_appearance_mode(self, mode_string):
-        if mode_string.lower() == "dark":
-            self.appearance_mode = 1
-        elif mode_string.lower() == "light":
-            self.appearance_mode = 0
+        if "bg_color" in kwargs:
+            self.bg_color = kwargs["bg_color"]
+            del kwargs["bg_color"]
+            require_redraw = True
 
-        if isinstance(self.master, CTkFrame):
-            self.bg_color = self.master.fg_color
-        else:
-            self.bg_color = self.master.cget("bg")
+        if "fg_color" in kwargs:
+            self.fg_color = kwargs["fg_color"]
+            del kwargs["fg_color"]
+            require_redraw = True
 
-        self.draw()
+        if "text_color" in kwargs:
+            self.text_color = kwargs["text_color"]
+            del kwargs["text_color"]
+            require_redraw = True
+
+        if "corner_radius" in kwargs:
+            self.corner_radius = self.calc_optimal_corner_radius(kwargs["corner_radius"])  # optimise for less artifacts
+
+            if self.corner_radius * 2 > self.height:
+                self.corner_radius = self.height / 2
+            elif self.corner_radius * 2 > self.width:
+                self.corner_radius = self.width / 2
+
+            self.entry.grid(column=0, row=0, sticky="we", padx=self.corner_radius if self.corner_radius >= 6 else 6)
+            del kwargs["corner_radius"]
+            require_redraw = True
+
+        self.entry.configure(*args, **kwargs)
+
+        if require_redraw is True:
+            self.draw()
 
     def delete(self, *args, **kwargs):
         return self.entry.delete(*args, **kwargs)
@@ -164,3 +224,15 @@ class CTkEntry(tkinter.Frame):
     def get(self):
         return self.entry.get()
 
+    def change_appearance_mode(self, mode_string):
+        if mode_string.lower() == "dark":
+            self.appearance_mode = 1
+        elif mode_string.lower() == "light":
+            self.appearance_mode = 0
+
+        if isinstance(self.master, (CTkFrame, CTk)):
+            self.bg_color = self.master.fg_color
+        else:
+            self.bg_color = self.master.cget("bg")
+
+        self.draw()
