@@ -1,32 +1,56 @@
 import tkinter
 import sys
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
 
 
 class CTkLabel(tkinter.Frame):
-    def __init__(self,
+    def __init__(self, *args,
                  master=None,
                  bg_color=None,
-                 fg_color=None,
-                 text_color=CTkColorManager.TEXT,
+                 fg_color="CTkColorManager",
+                 text_color="CTkColorManager",
                  corner_radius=8,
                  width=120,
                  height=25,
                  text="CTkLabel",
                  text_font=None,
-                 *args,
                  **kwargs):
-        super().__init__(master=master)
+        if master is None:
+            super().__init__(*args)
+        else:
+            super().__init__(*args, master=master)
 
-        AppearanceModeTracker.add(self.change_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribut>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.change_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.fg_color = self.bg_color if fg_color is None else fg_color
-        self.text_color = text_color
+        self.fg_color = CTkColorManager.LABEL_BG if fg_color == "CTkColorManager" else fg_color
+        self.text_color = CTkColorManager.TEXT if text_color == "CTkColorManager" else text_color
 
         self.width = width
         self.height = height
@@ -49,7 +73,6 @@ class CTkLabel(tkinter.Frame):
                 self.text_font = ("TkDefaultFont", 10)
         else:
             self.text_font = text_font
-        self.configure(width=self.width, height=self.height)
 
         self.canvas = tkinter.Canvas(master=self,
                                      highlightthicknes=0,
@@ -62,11 +85,14 @@ class CTkLabel(tkinter.Frame):
                                         bd=0,
                                         text=self.text,
                                         font=self.text_font,
-                                        *args, **kwargs)
+                                        **kwargs)
         self.text_label.place(relx=0.5, rely=0.5, anchor=tkinter.CENTER)
 
         self.fg_parts = []
 
+        super().configure(width=self.width, height=self.height)
+
+        self.bind('<Configure>', self.update_dimensions)
         self.draw()
 
     def destroy(self):
@@ -94,6 +120,15 @@ class CTkLabel(tkinter.Frame):
             else:
                 return user_corner_radius
 
+    def update_dimensions(self, event):
+        # only redraw if dimensions changed (for performance)
+        if self.width != event.width or self.height != event.height:
+            self.width = event.width
+            self.height = event.height
+
+            self.canvas.config(width=self.width, height=self.height)
+            self.draw()
+
     def draw(self):
         self.canvas.delete("all")
         self.fg_parts = []
@@ -131,21 +166,26 @@ class CTkLabel(tkinter.Frame):
         else:
             self.canvas.configure(bg=self.bg_color)
 
+        if self.fg_color is None:
+            new_fg_color = self.bg_color
+        else:
+            new_fg_color = self.fg_color
+
         for part in self.fg_parts:
-            if type(self.fg_color) == tuple:
-                self.canvas.itemconfig(part, fill=self.fg_color[self.appearance_mode], width=0)
+            if type(new_fg_color) == tuple:
+                self.canvas.itemconfig(part, fill=new_fg_color[self.appearance_mode], width=0)
             else:
-                self.canvas.itemconfig(part, fill=self.fg_color, width=0)
+                self.canvas.itemconfig(part, fill=new_fg_color, width=0)
 
         if type(self.text_color) == tuple:
             self.text_label.configure(fg=self.text_color[self.appearance_mode])
         else:
             self.text_label.configure(fg=self.text_color)
 
-        if type(self.fg_color) == tuple:
-            self.text_label.configure(bg=self.fg_color[self.appearance_mode])
+        if type(new_fg_color) == tuple:
+            self.text_label.configure(bg=new_fg_color[self.appearance_mode])
         else:
-            self.text_label.configure(bg=self.fg_color)
+            self.text_label.configure(bg=new_fg_color)
 
     def config(self, *args, **kwargs):
         self.configure(*args, **kwargs)
@@ -175,7 +215,7 @@ class CTkLabel(tkinter.Frame):
             require_redraw = True
             del kwargs["text_color"]
 
-        super().configure(*args, **kwargs)
+        self.text_label.configure(*args, **kwargs)
 
         if require_redraw:
             self.draw()
@@ -190,7 +230,7 @@ class CTkLabel(tkinter.Frame):
         elif mode_string.lower() == "light":
             self.appearance_mode = 0
 
-        if isinstance(self.master, CTkFrame):
+        if isinstance(self.master, (CTkFrame, CTk)):
             self.bg_color = self.master.fg_color
         else:
             self.bg_color = self.master.cget("bg")
