@@ -1,6 +1,7 @@
 import sys
 import tkinter
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
@@ -9,24 +10,50 @@ from .customtkinter_color_manager import CTkColorManager
 class CTkProgressBar(tkinter.Frame):
     """ tkinter custom progressbar, always horizontal, values are from 0 to 1 """
 
-    def __init__(self,
+    def __init__(self, *args,
+                 variable=None,
                  bg_color=None,
-                 border_color=CTkColorManager.PROGRESS_BG,
-                 fg_color=CTkColorManager.PROGRESS_BG,
-                 progress_color=CTkColorManager.MAIN,
+                 border_color="CTkColorManager",
+                 fg_color="CTkColorManager",
+                 progress_color="CTkColorManager",
                  width=160,
                  height=10,
                  border_width=0,
-                 *args, **kwargs):
+                 **kwargs):
         super().__init__(*args, **kwargs)
 
-        AppearanceModeTracker.add(self.change_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribut>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.change_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.border_color = border_color
-        self.fg_color = fg_color
-        self.progress_color = CTkColorManager.MAIN if progress_color is None else progress_color
+        self.border_color = CTkColorManager.PROGRESS_BG if border_color == "CTkColorManager" else border_color
+        self.fg_color = CTkColorManager.PROGRESS_BG if fg_color == "CTkColorManager" else fg_color
+        self.progress_color = CTkColorManager.MAIN if progress_color == "CTkColorManager" else progress_color
+
+        self.variable = variable
+        self.variable_callback_blocked = False
+        self.variabel_callback_name = None
 
         self.width = width
         self.height = self.calc_optimal_height(height)
@@ -41,13 +68,20 @@ class CTkProgressBar(tkinter.Frame):
                                      height=self.height)
         self.canvas.place(x=0, y=0)
 
-        self.draw()
+        self.draw()  # initial draw
 
-        # set progress
-        self.set(self.value)
+        if self.variable is not None:
+            self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+            self.variable_callback_blocked = True
+            self.set(self.variable.get(), from_variable_callback=True)
+            self.variable_callback_blocked = False
 
     def destroy(self):
         AppearanceModeTracker.remove(self.change_appearance_mode)
+
+        if self.variable is not None:
+            self.variable.trace_remove("write", self.variabel_callback_name)
+
         super().destroy()
 
     def detect_color_of_master(self):
@@ -104,6 +138,7 @@ class CTkProgressBar(tkinter.Frame):
             self.canvas.itemconfig("border_line_1",
                                    capstyle=tkinter.ROUND,
                                    width=self.height + width_reduced)
+            self.canvas.lower("border_parts")
 
         # create inner button parts
         if not self.canvas.find_withtag("inner_parts"):
@@ -201,7 +236,58 @@ class CTkProgressBar(tkinter.Frame):
                             self.height / 2 + (self.width - self.height) * self.value + self.height / 2 - self.border_width + oval_bottom_right_shift,
                             self.height - self.border_width + oval_bottom_right_shift))
 
-    def set(self, value):
+    def configure(self, *args, **kwargs):
+        require_redraw = False  # some attribute changes require a call of self.draw() at the end
+
+        if "bg_color" in kwargs:
+            self.bg_color = kwargs["bg_color"]
+            del kwargs["bg_color"]
+            require_redraw = True
+
+        if "fg_color" in kwargs:
+            self.fg_color = kwargs["fg_color"]
+            del kwargs["fg_color"]
+            require_redraw = True
+
+        if "border_color" in kwargs:
+            self.border_color = kwargs["border_color"]
+            del kwargs["border_color"]
+            require_redraw = True
+
+        if "progress_color" in kwargs:
+            self.progress_color = kwargs["progress_color"]
+            del kwargs["progress_color"]
+            require_redraw = True
+
+        if "border_width" in kwargs:
+            self.border_width = kwargs["border_width"]
+            del kwargs["border_width"]
+            require_redraw = True
+
+        if "variable" in kwargs:
+            if self.variable is not None:
+                self.variable.trace_remove("write", self.variabel_callback_name)
+
+            self.variable = kwargs["variable"]
+
+            if self.variable is not None and self.variable != "":
+                self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+                self.set(self.variable.get(), from_variable_callback=True)
+            else:
+                self.variable = None
+
+            del kwargs["variable"]
+
+        super().configure(*args, **kwargs)
+
+        if require_redraw is True:
+            self.draw()
+
+    def variable_callback(self, var_name, index, mode):
+        if not self.variable_callback_blocked:
+            self.set(self.variable.get(), from_variable_callback=True)
+
+    def set(self, value, from_variable_callback=False):
         self.value = value
 
         if self.value > 1:
@@ -211,6 +297,11 @@ class CTkProgressBar(tkinter.Frame):
 
         self.draw(no_color_updates=True)
 
+        if self.variable is not None and not from_variable_callback:
+            self.variable_callback_blocked = True
+            self.variable.set(round(self.value) if isinstance(self.variable, tkinter.IntVar) else self.value)
+            self.variable_callback_blocked = False
+
     def change_appearance_mode(self, mode_string):
         if mode_string.lower() == "dark":
             self.appearance_mode = 1
