@@ -1,6 +1,7 @@
 import tkinter
 import sys
 
+from .customtkinter_tk import CTk
 from .customtkinter_frame import CTkFrame
 from .appearance_mode_tracker import AppearanceModeTracker
 from .customtkinter_color_manager import CTkColorManager
@@ -9,13 +10,13 @@ from .customtkinter_color_manager import CTkColorManager
 class CTkSlider(tkinter.Frame):
     """ tkinter custom slider, always horizontal """
 
-    def __init__(self,
+    def __init__(self, *args,
                  bg_color=None,
                  border_color=None,
-                 fg_color=CTkColorManager.SLIDER_BG,
-                 progress_color=None,
-                 button_color=CTkColorManager.MAIN,
-                 button_hover_color=CTkColorManager.MAIN_HOVER,
+                 fg_color="CTkColorManager",
+                 progress_color="CTkColorManager",
+                 button_color="CTkColorManager",
+                 button_hover_color="CTkColorManager",
                  from_=0,
                  to=1,
                  number_of_steps=None,
@@ -23,23 +24,44 @@ class CTkSlider(tkinter.Frame):
                  height=16,
                  border_width=5,
                  command=None,
-                 *args, **kwargs):
+                 variable=None,
+                 **kwargs):
         super().__init__(*args, **kwargs)
 
-        AppearanceModeTracker.add(self.change_appearance_mode)
+        # overwrite configure methods of master when master is tkinter widget, so that bg changes get applied on child CTk widget too
+        if isinstance(self.master, (tkinter.Tk, tkinter.Frame)) and not isinstance(self.master, (CTk, CTkFrame)):
+            master_old_configure = self.master.config
+
+            def new_configure(*args, **kwargs):
+                if "bg" in kwargs:
+                    self.configure(bg_color=kwargs["bg"])
+                elif "background" in kwargs:
+                    self.configure(bg_color=kwargs["background"])
+
+                # args[0] is dict when attribute gets changed by widget[<attribut>] syntax
+                elif len(args) > 0 and type(args[0]) == dict:
+                    if "bg" in args[0]:
+                        self.configure(bg_color=args[0]["bg"])
+                    elif "background" in args[0]:
+                        self.configure(bg_color=args[0]["background"])
+                master_old_configure(*args, **kwargs)
+
+            self.master.config = new_configure
+            self.master.configure = new_configure
+
+        AppearanceModeTracker.add(self.change_appearance_mode, self)
         self.appearance_mode = AppearanceModeTracker.get_mode()  # 0: "Light" 1: "Dark"
 
         self.bg_color = self.detect_color_of_master() if bg_color is None else bg_color
-        self.border_color = self.bg_color if border_color is None else border_color
-        self.fg_color = fg_color
-        self.progress_color = progress_color if progress_color is not None else fg_color
-        self.button_color = self.bg_color if button_color is None else button_color
-        self.button_hover_color = self.bg_color if button_hover_color is None else button_hover_color
+        self.border_color = border_color
+        self.fg_color = CTkColorManager.SLIDER_BG if fg_color == "CTkColorManager" else fg_color
+        self.progress_color = CTkColorManager.SLIDER_PROGRESS if progress_color == "CTkColorManager" else progress_color
+        self.button_color = CTkColorManager.MAIN if button_color == "CTkColorManager" else button_color
+        self.button_hover_color = CTkColorManager.MAIN_HOVER if button_hover_color == "CTkColorManager" else button_hover_color
 
         self.width = width
         self.height = self.calc_optimal_height(height)
         self.border_width = round(border_width)
-        self.callback_function = command
         self.value = 0.5  # initial value of slider in percent
         self.hover_state = False
         self.from_ = from_
@@ -47,6 +69,11 @@ class CTkSlider(tkinter.Frame):
         self.number_of_steps = number_of_steps
         self.output_value = self.from_ + (self.value * (self.to - self.from_))
 
+        self.callback_function = command
+        self.variable: tkinter.Variable = variable
+        self.variable_callback_blocked = False
+        self.variabel_callback_name = None
+
         self.configure(width=self.width, height=self.height)
         if sys.platform == "darwin":
             self.configure(cursor="pointinghand")
@@ -62,10 +89,22 @@ class CTkSlider(tkinter.Frame):
         self.canvas.bind("<Button-1>", self.clicked)
         self.canvas.bind("<B1-Motion>", self.clicked)
 
-        self.draw()
+        self.draw()  # initial draw
+
+        if self.variable is not None:
+            self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+            self.variable_callback_blocked = True
+            self.set(self.variable.get(), from_variable_callback=True)
+            self.variable_callback_blocked = False
 
     def destroy(self):
+        # remove change_appearance_mode function from callback list of AppearanceModeTracker
         AppearanceModeTracker.remove(self.change_appearance_mode)
+
+        # remove variabel_callback from variable callbacks if variable exists
+        if self.variable is not None:
+            self.variable.trace_remove("write", self.variabel_callback_name)
+
         super().destroy()
 
     def detect_color_of_master(self):
@@ -99,9 +138,19 @@ class CTkSlider(tkinter.Frame):
 
         if no_color_updates is False:
             self.canvas.configure(bg=CTkColorManager.single_color(self.bg_color, self.appearance_mode))
-            self.canvas.itemconfig("border_parts", fill=CTkColorManager.single_color(self.border_color, self.appearance_mode))
+
+            if self.border_color is None:
+                self.canvas.itemconfig("border_parts", fill=CTkColorManager.single_color(self.bg_color, self.appearance_mode))
+            else:
+                self.canvas.itemconfig("border_parts", fill=CTkColorManager.single_color(self.border_color, self.appearance_mode))
+
             self.canvas.itemconfig("inner_parts", fill=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
-            self.canvas.itemconfig("progress_parts", fill=CTkColorManager.single_color(self.progress_color, self.appearance_mode))
+
+            if self.progress_color is None:
+                self.canvas.itemconfig("progress_parts", fill=CTkColorManager.single_color(self.fg_color, self.appearance_mode))
+            else:
+                self.canvas.itemconfig("progress_parts", fill=CTkColorManager.single_color(self.progress_color, self.appearance_mode))
+
             self.canvas.itemconfig("button_parts", fill=CTkColorManager.single_color(self.button_color, self.appearance_mode))
 
     def draw_with_polygon_shapes(self):
@@ -136,7 +185,7 @@ class CTkSlider(tkinter.Frame):
             self.canvas.delete("progress_parts")
 
         self.canvas.coords("inner_line_1",
-                           (self.height / 2,
+                           (((self.width + coordinate_shift - self.height) * self.value + self.height / 2),
                             self.height / 2,
                             self.width - self.height / 2 + coordinate_shift,
                             self.height / 2))
@@ -260,6 +309,11 @@ class CTkSlider(tkinter.Frame):
         if self.callback_function is not None:
             self.callback_function(self.output_value)
 
+        if self.variable is not None:
+            self.variable_callback_blocked = True
+            self.variable.set(round(self.output_value) if isinstance(self.variable, tkinter.IntVar) else self.output_value)
+            self.variable_callback_blocked = False
+
     def on_enter(self, event=0):
         self.hover_state = True
         self.canvas.itemconfig("button_parts", fill=CTkColorManager.single_color(self.button_hover_color, self.appearance_mode))
@@ -279,7 +333,7 @@ class CTkSlider(tkinter.Frame):
     def get(self):
         return self.output_value
 
-    def set(self, output_value):
+    def set(self, output_value, from_variable_callback=False):
         if output_value > self.to:
             output_value = self.to
         elif output_value < self.from_:
@@ -293,6 +347,15 @@ class CTkSlider(tkinter.Frame):
         if self.callback_function is not None:
             self.callback_function(self.output_value)
 
+        if self.variable is not None and not from_variable_callback:
+            self.variable_callback_blocked = True
+            self.variable.set(round(self.output_value) if isinstance(self.variable, tkinter.IntVar) else self.output_value)
+            self.variable_callback_blocked = False
+
+    def variable_callback(self, var_name, index, mode):
+        if not self.variable_callback_blocked:
+            self.set(self.variable.get(), from_variable_callback=True)
+
     def config(self, *args, **kwargs):
         self.configure(*args, **kwargs)
 
@@ -356,6 +419,20 @@ class CTkSlider(tkinter.Frame):
             self.callback_function = kwargs["command"]
             del kwargs["command"]
 
+        if "variable" in kwargs:
+            if self.variable is not None:
+                self.variable.trace_remove("write", self.variabel_callback_name)
+
+            self.variable = kwargs["variable"]
+
+            if self.variable is not None and self.variable != "":
+                self.variabel_callback_name = self.variable.trace_add("write", self.variable_callback)
+                self.set(self.variable.get(), from_variable_callback=True)
+            else:
+                self.variable = None
+
+            del kwargs["variable"]
+
         super().configure(*args, **kwargs)
 
         if require_redraw:
@@ -373,4 +450,3 @@ class CTkSlider(tkinter.Frame):
             self.bg_color = self.master.cget("bg")
 
         self.draw()
-
